(defun test-web-search ()
  (format t "~%WEB-SEARCH-TEST~%")
  (let ((web-search (first 
		     (yahoo:ysearch :web :query "lisp" :type 'any 
				    :results 2 :start 3 :adult_ok 1))))
    (format t "~%Title: ~a~%Summary: ~a~%Url: ~a~%ClickUrl: ~a~%MimeType: ~a~%ModificationDate: ~a~%"
	    (yahoo:title web-search)
	    (yahoo:summary web-search)
	    (yahoo:url web-search)
	    (yahoo:click-url web-search)
	    (yahoo:mime-type web-search)
	    (yahoo:modification-date web-search))))

(defun test-image-search ()
  (format t "~%IMAGE-SEARCH-TEST~%")
  (let ((image-search (first
		       (yahoo:ysearch :image :query "lisp" :type 'any
				      :results 2 :start 3 :format 'jpg))))
    (format t "~%Title: ~a~%Summary: ~a~%Url: ~a~%ClickUrl: ~a~%Referer: ~a~%FileSize: ~a~%FileFormat: ~a~%Height: ~a~%Width: ~a~%Publisher: ~a~%Restrictions: ~a~%CopyRight: ~a~%"
	    (yahoo:title image-search)
	    (yahoo:summary image-search)
	    (yahoo:url image-search)
	    (yahoo:click-url image-search)
	    (yahoo:referer-url image-search)
	    (yahoo:file-size image-search)
	    (yahoo:file-format image-search)
	    (yahoo:height image-search)
	    (yahoo:width image-search)
	    (yahoo:publisher image-search)
	    (yahoo:restrictions image-search)
	    (yahoo:copyright image-search))))

(defun test-spelling ()
  (format t "~%SPELLING-TEST~%")
  (let ((word "improbabble"))
    (format t "~%The correct spelling of ~a is ~a.~%" word (yahoo:result (first (yahoo:ysearch :spelling :query word))))))

(defun test-news-search ()
  (format t "~%NEWS-SEARCH-TEST~%")
  (let ((news-search (first 
		      (yahoo:ysearch :news :query "lisp" :sort 'date 
				     :results 3 :language 'en))))
    (format t "~%Title: ~a~%Summary: ~a~%Url: ~a~%ClickUrl: ~a~%NewsSource: ~a~%NewsSourceUrl: ~a~%Language: ~a~%PublishDate: ~a~%ModificationDate: ~a~%"
	    (yahoo:title news-search)
	    (yahoo:summary news-search)
	    (yahoo:url news-search)
	    (yahoo:click-url news-search)
	    (yahoo:news-source news-search)
	    (yahoo:news-source-url news-search)
	    (yahoo:language news-search)
	    (yahoo:publish-date news-search)
	    (yahoo:modification-date news-search))))

(defun test-video-search ()
  (format t "~%VIDEO-SEARCH-TEST~%")
  (let ((video-search (first
		       (yahoo:ysearch :video :query "hendrix" :results 3 :format 'mpeg))))
    (format t "~%Title: ~a~%Summary: ~a~%Url: ~a~%ClickUrl: ~a~%Referer: ~a~%FileSize: ~a~%FileFormat: ~a~%Height: ~a~%Width: ~a~%Duration: ~a~%Channels: ~a~%Streaming: ~a~%Publisher: ~a~%Restrictions: ~a~%CopyRight: ~a~%"
	    (yahoo:title video-search)
	    (yahoo:summary video-search)
	    (yahoo:url video-search)
	    (yahoo:click-url video-search)
	    (yahoo:referer-url video-search)
	    (yahoo:file-size video-search)
	    (yahoo:file-format video-search)
	    (yahoo:height video-search)
	    (yahoo:width video-search)
	    (yahoo:duration video-search)
	    (yahoo:channels video-search)
	    (yahoo:streaming video-search)
	    (yahoo:publisher video-search)
	    (yahoo:restrictions video-search)
	    (yahoo:copyright video-search))))

(defun test-local-search ()
  (format t "~%LOCAL-SEARCH-TEST~%")
  (let ((local-search (first (yahoo:ysearch :local :query "pizza" :results 2 :zip 94306))))
    (format t "~%Title: ~a~%City: ~a~%State: ~a~%Phone: ~a~%Rating: ~a~%Distance: ~a~%Url: ~a~%ClickUrl:: ~a~%MapUrl: ~a~%BusinessUrl: ~a~%BusinessClickUrl:: ~a~%"
	    (yahoo:title local-search)
	    (yahoo:city local-search)
	    (yahoo:state local-search)
	    (yahoo:phone local-search)
	    (yahoo:rating local-search)
	    (yahoo:distance local-search)
	    (yahoo:url local-search)
	    (yahoo:click-url local-search)
	    (yahoo:map-url local-search)
	    (yahoo:business-url local-search)
	    (yahoo:business-click-url local-search))))

(defun test-related ()
  (format t "~%RELATED SUGGESTIONS TEST~%")
  (let ((related-search (yahoo:ysearch :related :query "lisp" :results 8)))
    (dolist (result related-search)
      (format t "~%Related: ~S" (yahoo:result result)))))

(defun test-term-extraction ()
  (format t "~%~%TERM-EXTRACTION-TEST~%")
  (let* ((text "Italian sculptors and painters of the renaissance favored the Virgin Mary for inspiration.")
	 (extracted (yahoo:ysearch :term-extraction :context text :query "madonna")))
    (dolist (term extracted)
      (format t "~%Extracted term: ~S" (yahoo:result term)))))

(defun test-contextual-search ()
  (format t "~%~%CONTEXTUAL-SEARCH-TEST~%")
  (let* ((text "Italian sculptors and painters of the renaissance favored the Virgin Mary for inspiration.")
	 (result (first (yahoo:ysearch :contextual
				       :context text
				       :query "madonna"
				       :results 3))))
    (format t "~%Title: ~a~%Url: ~a"
	    (yahoo:title result)
	    (yahoo:url result))))

(defun test-errors ()
  (format t "~%~%ERROR TESTS~%")
  (let ((yahoo::*appid* ""))
    (handler-case (yahoo:ysearch :web :query "lisp")
      (yahoo:yahoo-error (condition)
	(format t "~%Yahoo reported the following error: ~%~S~%" 
		(yahoo::error-message condition))))
    (handler-case (yahoo:ysearch :loco :query "lisp")
      (yahoo:invalid-search-type-error (condition)
	(format t "~%Invalid search type error ~S" 
		(yahoo::search-type condition))))))

(defun test-all ()
  (progn 
    (test-web-search)
    (test-image-search)
    (test-news-search)
    (test-spelling)
    (test-video-search)
    (test-local-search)
    (test-related)
    (test-term-extraction)
    (test-contextual-search)
    (test-errors)))
