(in-package #:cl-yahoo)

;;;;; UTILITIES


(defmacro with-gensyms (syms &body body)
  `(let ,(mapcar #'(lambda (s)
                     `(,s (gensym)))
                 syms)
     ,@body))

(defun add-colon (symbol)
  (if (keywordp symbol)
      symbol
      (intern (symbol-name symbol) :keyword)))

(defun length=1 (list)
  "Is this a list of exactly one element?"
  (and (consp list) (null (cdr list))))

(defun get-search-url (type &optional (table *search-types*))
  (cdr (assoc type table)))

(defun make-assoc-list (alist)
  "Remove symbols from the top-level of an assoc list."
  (if (= (length alist) 2)
      (list alist)
      (remove-if-not #'consp alist)))

(defun strcat (&rest strings)
  (apply 'concatenate 'string strings))

(defun symcat (&rest symbols)
  (intern (apply 'strcat (mapcar 'symbol-name symbols))))

(defun slot-val-not-nil-p (obj slot-name)
  "Return T if slot exists and is not NIL."
  (and (slot-exists-p obj slot-name)
       (slot-boundp obj slot-name)
       (not (null (slot-value obj slot-name)))))

(defun fetch-and-parse-xml (url query &key http-post)
  "Fetch and parse an url into xml. 
   If HTTP-POST is T, do a HTTP-POST."
  (let ((stream (third (if http-post
			   (thttp:http-post url *content-type* query)
			   (thttp:http-get (strcat url query))))))
    (s-xml:parse-xml stream)))

(defun make-search-string (keys)
  "Return a query URL to be sent to Yahoo."
  (flet ((make-key/val-pair (key val)
	   (strcat "&" 
		   (string-downcase key) 
		   "=" 
		   (string-downcase (princ-to-string val)))))
    (loop for (key val) on keys by #'cddr
       with search-string =  (make-key/val-pair 'appid *appid*)
       when val
       do (setf search-string 
		(strcat search-string
			(make-key/val-pair 
			 key (thttp:escape-url-query (princ-to-string val)))))
       finally (return search-string))))

(defun lispify-var (symbol)
  "Convert a symbol to a more lispy form. 
   Ex. :|BusinessClickUrl| => BUSINESS-CLICK-URL"
  (unless (symbolp symbol)
    (error "~%~a is not a symbol." symbol))
  (let ((name (string-downcase (symbol-name symbol) :start 0 :end 1))
	(new-name (make-array '(0) :element-type 'character :adjustable t
			      :fill-pointer 0)))
    (macrolet ((sb-append (string char)
		 `(vector-push-extend 
		   (the character ,char) (the string ,string))))
      (loop for c across name 
	 if (upper-case-p c) do
	 (sb-append new-name #\-)
	 (sb-append new-name c)
	 else do
	 (sb-append new-name c))
      (intern (string-upcase new-name)))))
