(in-package #:cl-yahoo)


;;;;;;;;;; GENERIC FUNCTIONS

(defgeneric make-query-url (type &key &allow-other-keys))  

(defgeneric ysearch (type  &key &allow-other-keys)
  (:method ((type t)  &key &allow-other-keys)
    (error 'invalid-search-type-error :search-type type)))

;;;;;;;;;; CONDITIONS

(define-condition invalid-search-type-error (error)
  ((search-type :reader search-type :initarg :search-type))
  (:report (lambda (condition stream)
	     (let ((type (search-type condition)))
	       (if (keywordp type)
		   (format stream "~%~S is not a valid search type." type)
		   (format stream "~%Search type must be a keyword (~s). "
			   (intern (string type) :keyword)))))))

(define-condition yahoo-error (error) 
  ((error-message :reader error-message :initarg :error-message))
  (:report (lambda (condition stream)
	     (format stream "~%Yahoo reports the following error: ~%~a."
		     (error-message condition)))))

;;;;;;;;;; SEARCH

;;;;; WEB

(define-ysearch :type :web
  :request-parameters	(query type results start format adult_ok similar_ok 
			       language country license)
  :result-fields (:|Title| :|Summary| :|Url| :|ClickUrl| :|MimeType|
		   :|ModificationDate|))

;;;;; IMAGE

(define-ysearch :type :image
  :request-parameters (query type results start format adult_ok coloration)
  :result-fields (:|Title| :|Summary| :|Url| :|ClickUrl| :|RefererUrl|
		   :|FileSize| :|FileFormat| :|Height| :|Width| 
		   :|Publisher| :|Restrictions| :|Copyright|))

;;;;; NEWS

(define-ysearch :type :news
  :request-parameters (query type results start sort language)
  :result-fields  (:|Title| :|Summary| :|Url| :|ClickUrl| :|NewsSource| 
		    :|NewsSourceUrl| :|Language| :|PublishDate| 
		    :|ModificationDate|))

;;;;; VIDEO  

(define-ysearch :type :video
  :request-parameters (query type results start format adult_ok)
  :result-fields  (:|Title| :|Summary| :|Url| :|ClickUrl| :|RefererUrl| 
		    :|FileSize|	:|FileFormat| :|Height| :|Width| :|Duration|
		    :|Channels| :|Streaming| :|Publisher| :|Restrictions| 
		    :|Copyright|))

;;;;; LOCAL  

(define-ysearch :type :local
  :request-parameters (query type results start sort radius street city
			     state zip location latitude longitude)
  :result-fields (:|Title| :|City| :|State| :|Phone| :|Rating|
		   :|Distance| :|Url| :|ClickUrl| :|MapUrl| :|BusinessUrl| 
		   :|BusinessClickUrl|))

;;;;; RELATED SUGGESTIONS  

(define-ysearch :type :related
  :request-parameters (query results)
  :result-fields (:|Result|)
  :print-method result)

;;;;; SPELLING 

(define-ysearch :type :spelling
  :request-parameters (query results)
  :result-fields (:|Result|)
  :print-method result)

;;;;; TERM-EXTRACTION 

(define-ysearch :type :term-extraction
  :request-parameters (query context)
  :result-fields (:|Result|)
  :print-method result
  :http-post t)

;;;;; CONTEXTUAL WEB SEARCH

(define-ysearch :type :contextual
  :request-parameters (context query type results start format adult_ok
			       similar_ok language country license)
  :result-fields (:|Result| :|Title| :|Summary| :|Url| :|ClickUrl|
		   :|MimeType| :|ModificationDate|)
  ;; should add :|Cache| to the above, but not working currently
  ;; because of nested xml-fields in the cache-result. XXX
  :http-post t)
